@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import functools
 import json
 import logging
@@ -7,7 +9,7 @@ import socket
 import ssl
 import threading
 from asyncio.selector_events import BaseSelectorEventLoop
-from typing import Dict, List, Match, Optional, Union
+from typing import Dict, List, Match, Optional, Type, TypeVar, Union
 from urllib.parse import parse_qs, unquote, urlencode, urlparse
 
 import requests
@@ -27,12 +29,12 @@ from localstack.config import (
     EXTRA_CORS_ALLOWED_HEADERS,
     EXTRA_CORS_ALLOWED_ORIGINS,
     EXTRA_CORS_EXPOSE_HEADERS,
-    is_env_true,
 )
 from localstack.constants import (
     APPLICATION_JSON,
     AWS_REGION_US_EAST_1,
     BIND_HOST,
+    HEADER_LOCALSTACK_AUTHORIZATION,
     HEADER_LOCALSTACK_REQUEST_URL,
 )
 from localstack.services.messages import Headers, MessagePayload
@@ -42,18 +44,15 @@ from localstack.utils.aws import aws_stack
 from localstack.utils.aws.aws_responses import LambdaResponse
 from localstack.utils.aws.aws_stack import is_internal_call_context
 from localstack.utils.aws.request_context import RequestContextManager, get_proxy_request_for_thread
-from localstack.utils.common import (
-    empty_context_manager,
-    generate_ssl_cert,
-    json_safe,
-    path_from_url,
-    start_thread,
-    to_bytes,
-    to_str,
-    wait_for_port_open,
-)
+from localstack.utils.crypto import generate_ssl_cert
+from localstack.utils.functions import empty_context_manager
+from localstack.utils.json import json_safe
+from localstack.utils.net import wait_for_port_open
 from localstack.utils.server import http2_server
 from localstack.utils.serving import Server
+from localstack.utils.strings import to_bytes, to_str
+from localstack.utils.threads import start_thread
+from localstack.utils.urls import path_from_url
 
 # set up logger
 LOG = logging.getLogger(__name__)
@@ -347,17 +346,19 @@ class ArnPartitionRewriteListener(MessageModifyingProxyListener):
 # BASE BACKEND UTILS
 # -------------------
 
+T = TypeVar("T", bound="RegionBackend")
+
 
 class RegionBackend(object):
     """Base class for region-specific backends for the different APIs.
     RegionBackend lookup methods are not thread safe."""
 
-    REGIONS: Dict[str, "RegionBackend"]
+    REGIONS: Dict[str, T]
 
     name: str  # name of the region
 
     @classmethod
-    def get(cls, region: str = None) -> "RegionBackend":
+    def get(cls: Type[T], region: str = None) -> T:
         region = region or cls.get_current_request_region()
 
         regions = cls.regions()
@@ -370,7 +371,7 @@ class RegionBackend(object):
         return regions[region]
 
     @classmethod
-    def regions(cls) -> Dict[str, "RegionBackend"]:
+    def regions(cls: Type[T]) -> Dict[str, T]:
         if not hasattr(cls, "REGIONS"):
             # maps region name to region backend instance
             cls.REGIONS = {}
@@ -662,9 +663,10 @@ def modify_and_forward(
 
         # make sure we drop "chunked" transfer encoding from the headers to be forwarded
         headers_to_send.pop("Transfer-Encoding", None)
+
         response = requests.request(
             method_to_send,
-            request_url,
+            url=request_url,
             data=data_to_send,
             headers=headers_to_send,
             stream=True,
@@ -673,9 +675,8 @@ def modify_and_forward(
 
     # prevent requests from processing response body (e.g., to pass-through gzip encoded content
     # unmodified)
-    pass_raw = (
-        hasattr(response, "_content_consumed") and not response._content_consumed
-    ) or response.headers.get("content-encoding") in ["gzip"]
+    not_consumed = not getattr(response, "_content_consumed", True)
+    pass_raw = not_consumed or response.headers.get("content-encoding") in ["gzip"]
     if pass_raw and getattr(response, "raw", None):
         new_content = response.raw.read()
         if new_content:
@@ -947,6 +948,8 @@ def start_proxy_server(
     params=None,  # TODO: not being used - should be investigated/removed
     asynchronous=True,
     check_port=True,
+    max_content_length: int = None,
+    send_timeout: int = None,
 ):
     bind_address = bind_address if bind_address else BIND_HOST
 
@@ -963,8 +966,8 @@ def start_proxy_server(
         method = request.method
         headers = request.headers
         headers[HEADER_LOCALSTACK_REQUEST_URL] = str(request.url)
-
-        response = modify_and_forward(
+        headers[HEADER_LOCALSTACK_AUTHORIZATION] = headers.get("Authorization", "")
+        return modify_and_forward(
             method=method,
             path=path_with_params,
             data_bytes=data,
@@ -975,8 +978,6 @@ def start_proxy_server(
             server_address=parsed_url.netloc,
         )
 
-        return response
-
     ssl_creds = (None, None)
     if use_ssl:
         install_predefined_cert_if_available()
@@ -984,7 +985,13 @@ def start_proxy_server(
         ssl_creds = (cert_file_name, key_file_name)
 
     result = http2_server.run_server(
-        port, bind_address, handler=handler, asynchronous=asynchronous, ssl_creds=ssl_creds
+        port,
+        bind_address,
+        handler=handler,
+        asynchronous=asynchronous,
+        ssl_creds=ssl_creds,
+        max_content_length=max_content_length,
+        send_timeout=send_timeout,
     )
     if asynchronous and check_port:
         wait_for_port_open(port, sleep_time=0.2, retries=12)
@@ -995,7 +1002,7 @@ def install_predefined_cert_if_available():
     try:
         from localstack_ext.bootstrap import install
 
-        if is_env_true("SKIP_SSL_CERT_DOWNLOAD"):
+        if config.SKIP_SSL_CERT_DOWNLOAD:
             LOG.debug("Skipping download of local SSL cert, as SKIP_SSL_CERT_DOWNLOAD=1")
             return
         install.setup_ssl_cert()
